#include <mpi.h>

extern int squaresize[4];          /* dimensions of hypercubes */

MPI_Datatype fsu3_vector_mpi_t, fsu3_matrix_mpi_t, fhalf_wilson_vector_mpi_t;
MPI_Datatype dsu3_vector_mpi_t, dsu3_matrix_mpi_t, dhalf_wilson_vector_mpi_t;

MPI_Type_continguous(3, MPI_FLOAT, &fsu3_vector_mpi_t);
MPI_Type_continguous(6, MPI_FLOAT, &fhalf_wilson_vector_mpi_t);
MPI_Type_continguous(9, MPI_FLOAT, &fsu3_matrix_mpi_t);

MPI_Type_continguous(3, MPI_DOUBLE, &dsu3_vector_mpi_t);
MPI_Type_continguous(6, MPI_DOUBLE, &dhalf_wilson_vector_mpi_t);
MPI_Type_continguous(9, MPI_DOUBLE, &dsu3_matrix_mpi_t);

#if (PRECISION==1)

#define su3_vector_mpi_t fsu3_vector_mpi_t
#define half_wilson_vector_mpi_t fhalf_wilson_vector_mpi_t
#define su3_matrix_mpi_t fsu3_matrix_mpi_t

# else

#define su3_vector_mpi_t dsu3_vector_mpi_t
#define half_wilson_vector_mpi_t dhalf_wilson_vector_mpi_t
#define su3_matrix_mpi_t dsu3_matrix_mpi_t

#endif

MPI_Datatype field_neighbor_ddt[3][2][8];  /* 3 for su3_vector, half_wilson_vector	
						su3_matrix */
					   /* 2 for EVEN or EVENANDODD */
					   /* 8 for direction */
MPI_Datatype site_neighbor_ddt[3][2][8];

MPI_Type_contiguous(squaresize[XUP]*squaresize[YUP]*squaresize[ZUP]/2,
		su3_vector_mpi_t, &field_neighbor_ddt[0][0][TUP]);
MPI_type_hvector(2,1,sizeof(su3_vector)*even_sites_on_node,field_neighbor_ddt[0][0][TUP],
				&field_neighbor_ddt[0][1][TUP]
MPI_Type_contiguous(squaresize[XUP]*squaresize[YUP]*squaresize[ZUP]/2,
		half_wilson_vector_mpi_t, &field_neighbor_ddt[1][0][TUP]);
MPI_type_hvector(2,1,sizeof(half_wilson_vector)*even_sites_on_node,field_neighbor_ddt[1][0][TUP],
				&field_neighbor_ddt[1][1][TUP]
MPI_Type_contiguous(squaresize[XUP]*squaresize[YUP]*squaresize[ZUP]/2,
		su3_matrix_mpi_t, &field_neighbor_ddt[2][0][TUP]);
MPI_type_hvector(2,1,sizeof(su3_matrix)*even_sites_on_node,field_neighbor_ddt[2][0][TUP],
				&field_neighbor_ddt[2][1][TUP]

/* neighbors in ZUP direction */
MPI_Type_vector(squaresize[TUP], squaresize[XUP]*squaresize[YUP]/2,
		squaresize[XUP]*squaresize[YUP]*squaresize[ZUP]/2,
		su3_vector_mpi_t, &field_neighbor_ddt[0][0][ZUP]);
MPI_Type_vector(squaresize[TUP], squaresize[XUP]*squaresize[YUP]/2,
		squaresize[XUP]*squaresize[YUP]*squaresize[ZUP]/2,
		half_wilson_vector_mpi_t, &field_neighbor_ddt[1][0][ZUP]);
MPI_Type_vector(squaresize[TUP], squaresize[XUP]*squaresize[YUP]/2,
		squaresize[XUP]*squaresize[YUP]*squaresize[ZUP]/2,
		su3_matrix_mpi_t, &field_neighbor_ddt[2][0][ZUP]);
/* EVENANDODD */
MPI_type_hvector(2,1,sizeof(su3_vector)*even_sites_on_node,field_neighbor_ddt[0][0][ZUP],
				&field_neighbor_ddt[0][1][ZUP]
MPI_type_hvector(2,1,sizeof(half_wilson_vector)*even_sites_on_node,field_neighbor_ddt[1][0][ZUP],
				&field_neighbor_ddt[1][1][ZUP]
MPI_type_hvector(2,1,sizeof(su3_matrix)*even_sites_on_node,field_neighbor_ddt[2][0][ZUP],
				&field_neighbor_ddt[2][1][ZUP]

/* neighbors in YUP direction */
MPI_Type_vector(squaresize[ZUP]*squaresize[TUP], squaresize[XUP]/2,
		squaresize[XUP]*squaresize[YUP]/2,
		su3_vector_mpi_t, &field_neighbor_ddt[0][0][YUP]);
MPI_Type_vector(squaresize[ZUP]*squaresize[TUP], squaresize[XUP]/2,
		squaresize[XUP]*squaresize[YUP]/2,
		half_wilson_vector_mpi_t, &field_neighbor_ddt[1][0][YUP]);
MPI_Type_vector(squaresize[ZUP]*squaresize[TUP], squaresize[XUP]/2,
		squaresize[XUP]*squaresize[YUP]/2,
		su3_matrix_mpi_t, &field_neighbor_ddt[2][0][YUP]);
/* EVENANDODD */
MPI_type_hvector(2,1,sizeof(su3_vector)*even_sites_on_node,field_neighbor_ddt[0][0][YUP],
				&field_neighbor_ddt[0][1][YUP]
MPI_type_hvector(2,1,sizeof(half_wilson_vector)*even_sites_on_node,field_neighbor_ddt[1][0][YUP],
				&field_neighbor_ddt[1][1][YUP]
MPI_type_hvector(2,1,sizeof(su3_matrix)*even_sites_on_node,field_neighbor_ddt[2][0][YUP],
				&field_neighbor_ddt[2][1][YUP]

/* this next case is problematic!  Need to work on it */
MPI_Type_vector(squaresize[YUP]*squaresize[ZUP]*squaresize[TUP]/2, 1,
		squaresize[XUP]/2,
		su3_vector_mpi_t, &field_neighbor_ddt[0][0][XUP]);
